// OkolibExample.cpp 
//

#include "stdafx.h"
#include <iostream>
#include <ctime>
#include "okolib.h"
#include "windows.h"

int _tmain(int argc, _TCHAR* argv[])
{
	oko_res_type ret = OKO_ERR_UNINIT;
	char version[20] = "";
	char err_msg[500] = "";
	FILE *fp = NULL;
	uint32_t detected_num = 0;
	
	int cycle_num = 0;
	clock_t clock_start = clock();
	clock_t clock_cur = clock();
	double seconds = 0.0f;
		
	char name[20] = "";
	char dim_unit[20] = "";
	bool can_disable = false;
	double measure = 0.0f;
	double setpoint = 0.0f;

	bool enabled = false;
	double diffms = 0.0;

	/**** Init Okolib ****/
	ret = oko_LibInit("");
	if (ret != OKO_OK)
	{
		printf("Library cannot be initialized. Ret: %d\n", ret);
		
		oko_LibGetLastError(err_msg, 500);
		printf("Error: %s\n", err_msg);
		return 1;
	}
	
	/**** Get version ****/
	ret = oko_LibGetVersion(version);
	printf("LibGetVersion ::: version: %s ::: ret: %d\n", version, ret);

	/**** Create file ****/
	 errno_t ferr = fopen_s (&fp, "okolib.txt","w");
	 if(ferr != 0)
	 {
		 printf("Cannot create file. Err: %d\n", ferr);
	 }

	/**** Looking for connected modules ****/
	oko_module_type modules[OKO_MODULE_MAXNUM];

	ret = oko_ModulesDetect(OKO_MODULE_MAXNUM, modules, &detected_num);
	printf("oko_ModulesDetect ::: detected_num: %d ::: ret: %d\n", detected_num, (int) ret);

	/**** If at leaste a module has been found, start logging to file ***/
	while (detected_num)
	{
		clock_cur = clock();
		seconds = ((double)(clock_cur - clock_start))/CLOCKS_PER_SEC;

		cycle_num++;

		/*** Iterate on modules ****/
		for (unsigned int i = 0; i < detected_num; i++)
		{
			if (cycle_num <= 1)
			{
				/*! Read module infos */
				ret = oko_ModuleGetDetails(modules[i], name, dim_unit,
						&can_disable);

				printf("oko_ModuleGetDetails. Name: %s. Unit: %s. CanDisable: %d. Ret: %d\n",
						name, dim_unit, can_disable, ret);
			}
			else
			{
				if(i == 0)
				{
					fprintf(fp,"%.2f|",seconds);
				}
				else
					fprintf(fp,"|",seconds);

				/*** Get main value ***/
				ret = oko_ModuleGetCurrentValue(modules[i], &measure);
				if (ret != OKO_OK)
				{
					measure = -999.0;
				}

				/*** Get setpoint value ***/
				ret = oko_ModuleGetSetpointValue(modules[i], &setpoint);
				if (ret != OKO_OK)
				{
					setpoint = -999.0;
				}

				/*** Get enabled status****/
				ret = oko_ModuleGetEnabled(modules[i], &enabled);

				/**** Print infos in CSV format ***/
				fprintf(fp,"%.1f|%.1f|%d",measure,setpoint,enabled);

			}

		} //for

		/* Flush the file */
		fprintf(fp,"\n");
		fflush(fp);

		/*** Modules are queried each 500ms ***/
		diffms = (clock() - clock_cur)/(CLOCKS_PER_SEC/1000);
		if(diffms < 500)
		{
			Sleep(500 - (int)diffms);
		}

	} //while(1)

	printf("No modules found.");

	oko_LibShutDown();

	return 0;
}

